define(['jquery', 'bus'], function($, Bus){

	var bus = Bus.get_bus();

	function DashboardSetTestable(o){
		for(var i in o) this[i] = o[i];
		this.test = null;
		//this.set = $('.dashboard'); //too early
		this.n_dashboards = function(){
			return $('.dashboard ul.ui-tabs-nav > li').length;
		}
		this.change_dashboard = function(name){
			var self = this;
			return $.Deferred(function(df){
				var link = '.ui-tabs-nav a[href="#' + name + '"]';
				self.test.assert($(link).length === 1, 'bad dashboard name: ' + name);
				$(link).click();
				self.test.wait_for(function(){ return $(link).parent().hasClass('ui-state-active'); }, 'tab active').then(function(){
					df.resolve(self.test);
				});
			}).promise();
		}
		this.add_dashboard = function(name){
			var self = this;
			return $.Deferred(function(df){
				var dashboard;
				var n = self.n_dashboards();
				self.test.assert(self.n_dashboards() > 0, 'n_dashboards > 0');
				var a = $.Deferred(function(d){
					self.test._listen_once('dashboard-set-changed')._wait(function(){
						console.log('DashboardSetTestable: test A complete');
						self.test.assert(self.n_dashboards() === n + 1, 'n_dashboards: ' + self.n_dashboards() + ' != ' + (n + 1));
						d.resolve();
					});
				});
				var b = $.Deferred(function(d){
					$('.dashboard').dashboardSet('addTab', function(_dashboard){
						console.log('DashboardSetTestable.add_dashboard: B done', _dashboard);
						dashboard = $('#' + _dashboard.name).data('uiDashboard');

						self.test.assert(dashboard._active() === true, "active not set:", dashboard.active); //check the active flag is set correctly

						var dashboard_set = dashboard.dashboard_set_;
						var dashboards = dashboard_set.dashboards;
						var names = $.map(dashboards, function(v){ return v.options.dashboard.name; });
						self.test.assert(!!~$.inArray(_dashboard.name, names));

						d.resolve();
					});
				});
				$.when(a.promise(), b.promise()).then(function(){ df.resolveWith(self.test, [dashboard]); })
			}).promise();
		}
		this.remove_dashboard = function(name){
			var self = this;
			return $.Deferred(function(df){
				var n = self.n_dashboards();
				var dashboard = $('#' + name).dashboard('instance');
				bus.clear('dashboard-set-changed').listen_once('dashboard-set-changed', function(){
					setTimeout(function(){
						self.test.assert(self.n_dashboards() == n - 1, 'n_dashboards=' + self.n_dashboards() + ' expected ' + (n - 1));
						self.test.assert(!~$.inArray(name, $.map(dashboard.dashboard_set_.dashboards, function(v){ return v.options.dashboard.name; })), 'removed dashboard still in dashboard list');
						df.resolveWith(self.test);
					}, 1000); //wait for animations to finish.
				});
				$('.dashboard').dashboardSet('removeTab', name);
			}).promise();
		}
		this.move_widget = function(widget, from, to){
			console.log('----------------------------------');
			console.log('dst.move_widget: widget=', widget, from.dashboard2.options.name, '  --> ', to.dashboard2.options.name);
			console.log('dst.move_widget: widget=', widget, from.dashboard2.options.title, '  --> ', to.dashboard2.options.title);
			var self = this;
			var set = $('.dashboard').data('ui-dashboardSet');
			return $.Deferred(function(df){
				var n_src_widgets = from.n_widgets();
				var n_dest_widgets = to.dashboard2.widgets.count();
				from.$dashboard.prependTo('#' + to.dashboard.name + ' ul:first > li:first'); //emulate drag and drop
				from.$dashboard.dashboard('move_widget_to_other_tab', widget.options.name, to.dashboard.name, function(){
					self.test.wait_for(
						function(){ return from.n_widgets() === n_src_widgets - 1; },
						function(){ return 'src widget count: ' + from.n_widgets() + ' expected: ' + (n_src_widgets - 1) }
					).then(function(){
						self.test.assert( set.get_selected_dashboard().name === from.dashboard.name, 'selected-dashboard should be src dashboard');
						self.test.assert( from.$dashboard.data('uiDashboard')._active() === true,    'active should be set on src dashboard:', from.$dashboard.data('uiDashboard').active); //check the active flag is set correctly
						self.test.assert( to.$dashboard.data('uiDashboard')._active() === false,     'active should NOT be set on dest dashboard:', to.$dashboard.data('uiDashboard').active); //check the active flag is set correctly
						self.test.assert( to.dashboard2.widgets.count() === n_dest_widgets + 1, 'wrong widget count on destination: ' + to.dashboard2.widgets.count() + ' expected: ' + (n_dest_widgets + 1));

						df.resolveWith(self.test);
					});
				});
			}).promise();
		}
		this.ensure_interactive_chart = function(done){
			done()
		}
	}

	return DashboardSetTestable;
});
