define(['jquery', 'bus', '../widgets/widget'], function($, Bus, DashboardWidget){

	var bus = Bus.get_bus();

	function DashboardTestable(test){
		this.test = test;
		var dashboard_set = $('.dashboard').data('ui-dashboardSet');
		this.dashboard = dashboard_set.get_selected_dashboard(); // just get selected dashboard for now
		this.dashboard2 = dashboard_set.dashboard_object_lookup_by_name(this.dashboard.name);
		this.$dashboard = $('#' + this.dashboard.name);
		this.first_widget = $('body > .dashboardWidget:visible:first');

		this.n_widgets = function(){
			// returns the number of visible widgets (only widgets for the current dashboard should be visible)
			return $('body > .dashboardWidget:visible').length; // excludes any wrapped for slideup
		}

		this.check = function(){
			var n1 = $('body > .dashboardWidget:visible').length; // excludes any wrapped for slideup
			var n2 = this.dashboard2.widgets.count();
			console.log('      n=', n1, n2);

			if(n1 !== n2){
				test.fail('inconsistency in widget count: model= ' + n2 + ' visible=' + n1);
				this.dashboard2.widgets.all(function(v, c, r){
					console.log('  check: ' + c + '.' + r, v)
				})
				return false;
			}

			return true;
		}

		this.ensure_widget = function(done){
			var self = this;

			if(this.first_widget.length) return done(this.first_widget);

			this.add_widget(function(a){
				var widget = $('body > .dashboardWidget:visible:first');
				self.test.assert(!widget.find('.chart-controls').length, 'dashboard widget should not have controls');
				self.test.assert(widget.length === 1, 'dashboard has widget');

				done(widget);
			});
		}

		this.ensure_editable_widget = function(done){
			var self = this;

			test.assert(this.dashboard2.active, 'not active')

			var found = false;
			this.dashboard2.widgets.all(function(w, c, r){
				if(w.instance && w.instance._chart && w.instance._chart.product){
					var product = w.instance._chart.product
					if(product.is_eccharts()){
						done($('#' + w.widget.name))
						found = true;
						return false;
					}
				}
			})

			if(!found){
				this.add_widget(function(_, widget){
					self.test.assert(typeof widget !== undefined, 'got widget');

					test.wait_for(
						function(){ return widget.element.find('.widget-head .ui-icon-pencil').length === 1 },
						'pencil not found'
					).then(function(){
						done(widget.element);
					});
				});
			}
		}

		/*
		 *  Add an eccharts widget.
		 */
		this.add_widget = function(done){
			var self = this;
			self.test.set_timeout(10000);
			self.check();
			return $.Deferred(function(df){
				var widget;
				var n = self.n_widgets();
				var a = $.Deferred(function(d){
					bus.listen_once('dashboard-changed', function(){
						setTimeout(function(){
							test.assert(self.n_widgets() === n + 1, 'n_widgets: ' + self.n_widgets() + ' != ' + (n + 1));
							Test.wait_for(function(){
								return $('.chart-image:visible').length; // TODO add more checking here
							}).then(a.resolve);
						}, 1000);
					});
				});
				var b = $.Deferred(function(d){
					self.$dashboard.dashboard('add_widget', {widget: {kind: 'chart', data: {product: '2mT_wind30m'}}}, function(widget_name){
						widget = $('#' + widget_name).data('ui-chartDashboardWidget');
						if(widget){
							console.log('%cDashboardTestable: widget added:', 'color:#f00', widget_name, self.dashboard, widget);
							b.resolve();
						}else{
							console.error('widget added but data not found', $('#' + widget_name), $('#' + widget_name).data());
						}
					});
				});
				$.when(a.promise(), b.promise()).then(function(){ done && done(null, widget); df.resolveWith(self.test, [widget]); })
			}).promise();
		}

		this.remove_widget = function(widget_name){
			var self = this;
			var widget = typeof widget_name === 'string' ? $('#' + widget_name) : $('body > .dashboardWidget:visible:first');
			var done = typeof widget_name === 'function' ? widget_name : null;
			test.assert(widget.length === 1);
			self.check();
			test._set_timeout_info('remove widget');

			console.log('remove_widget: name=', widget.attr('id'));
			self.test.set_timeout(10000);
			return $.Deferred(function(df){
				var _n_widgets = self.n_widgets();
				widget.find('.ui-icon-close').click().orelse(function(){ test.fail('cannot find widget'); });
				setTimeout(function(){
					$('.alert #_ok').click();
					test.wait_for(function(){
						return (self.n_widgets() === _n_widgets - 1);
					}, function(){ return 'widget count: ' + self.n_widgets() + '!=' + (_n_widgets - 1) }).then(function(){
						self.check(); done && done(); df.resolveWith(self.test);
					});
				}, 1000)
			}).promise();
		}

		this.add_basic_widget = function(done){
			var self = this;
			self.test.set_timeout(10000);
			self.check();
			return $.Deferred(function(df){
				var widget;
				var n = self.n_widgets();
				var a = $.Deferred(function(d){
					bus.listen_once('dashboard-changed', function(){
						setTimeout(function(){
							test.assert(self.n_widgets() === n + 1, 'n_widgets: ' + self.n_widgets() + ' != ' + (n + 1));
							Test.wait_for(function(){
								return $('.chart-image:visible').length; // TODO add more checking here
							}).then(a.resolve);
						}, 1000);
					});
				});
				var b = $.Deferred(function(d){
					self.$dashboard.dashboard('add_widget', {widget: {kind: 'basic_chart', data: {product: 'public_plots'}}}, function(widget_name){
						widget = $('#' + widget_name).data('ui-basicChartDashboardWidget');
						if(widget){
							console.log('%cDashboardTestable: widget added:', 'color:#f00', widget_name, self.dashboard, widget);
							b.resolve();
						}else{
							console.error('widget added but data not found', $('#' + widget_name), $('#' + widget_name).data());
						}
					});
				});
				$.when(a.promise(), b.promise()).then(function(){ done && done(null, widget); df.resolveWith(self.test, [widget]); })
			}).promise();
		}

		this.add_interactive_catalogue_widget = function(done){
			var self = this;
			self.test.set_timeout(10000);
			self.check();
			return $.Deferred(function(df){
				var widget;
				var n = self.n_widgets();
				var a = $.Deferred(function(d){
					bus.listen_once('dashboard-changed', function(){
						setTimeout(function(){
							test.assert(self.n_widgets() === n + 1, 'n_widgets: ' + self.n_widgets() + ' != ' + (n + 1));
							Test.wait_for(function(){
								return $('.chart-image:visible').length; // TODO add more checking here
							}).then(a.resolve);
						}, 1000);
					});
				});
				var b = $.Deferred(function(d){
					self.$dashboard.dashboard('add_widget', {widget: {kind: 'chart', data: {product: 's2s_anomaly_cawcr_2MTM'}}}, function(widget_name){
						widget = $('#' + widget_name).data('ui-chartDashboardWidget');
						if(widget){
							console.log('%cDashboardTestable: widget added:', 'color:#f00', widget_name, self.dashboard, widget);
							b.resolve();
						}else{
							console.error('widget added but data not found', $('#' + widget_name), $('#' + widget_name).data());
						}
					});
				});
				$.when(a.promise(), b.promise()).then(function(){ done && done(null, widget); df.resolveWith(self.test, [widget]); })
			}).promise();
		}

		this.add_meteogram = function(widget_type){
			widget_type = widget_type || 'meteogram_10_days';
			var self = this;

			return $.Deferred(function(df){

				// temporarily using an internal method until the menu is added to the page
				var proto = DashboardWidget.registry()[widget_type]
				self.dashboard2.add_widget({widget: proto, top: true}, function(name){
					var widget = $('#' + name)
					test.assert(widget.length === 1);
					test.assert(widget.find('h3').text() === proto.title);
					df.resolveWith(self.test);
				})
			}).promise();
		}

		this.preview = function(widget_name){
			var self = this;
			var done = typeof widget_name === 'function' ? widget_name : null;

			var contents = {
				chart: '.chart',
				meteogram: '.content'
			}

			return $.Deferred(function(df){
				test.wait_for(function(){
					return $('body > .ui-dialog-preview:visible').length === 0;
				}, 'init')
				.then(function(){ 
					$('#' + widget_name + ' .widget-content').click();
					test
						.wait_for($.map(contents, function(c){ return '#preview ' + c + ':visible'; }).join(', '), 'preview visible')
						.then(function(){
							test.assert($('body > .ui-dialog-preview .ui-dialog-titlebar-close').length > 0, 'have close button');
							test.assert($('body > .ui-dialog-preview .time-nav').length > 0, 'have time navigator');
							test.assert($('body > .ui-dialog-preview .chart-controls > .bar-2').length == 1, 'single options menu');
							$('body > .ui-dialog-preview .ui-dialog-titlebar-close').click();
							test.wait_for(function(){
								return $('body > .ui-dialog-preview .content:visible').length === 0;
							}, 'preview closed')
							.then(function(){ done && done(); df.resolveWith(self.test); });
						});
				});
			}).promise();
		}

		/*
		 * open/close the preview window for a widget multiple times
		 */
		this.multiple_previews = function(widget_name){
			var self = this;

			if(self.test.running) return $.Deferred(function(df){
				$.when(self.preview(widget_name)).then(function(){
					$.when(self.preview(widget_name)).then(function(){ df.resolveWith(self.test); });
				});
			}).promise();
		}

		this.edit_start = function(widget_name){
			var self = this;

			var selector = '#' + widget_name + ' .widget-head .ui-icon-pencil'
			test.assert($(selector).length === 1, 'widget button not found: ' + selector)

			return $.Deferred(function(df){
				$('#' + widget_name + ' .widget-head .ui-icon-pencil').click();
				test
					.wait_for(function(){
						return $('#' + widget_name + ' .edit .form:visible').length;
					})
					.then(function(){
						df.resolveWith(self.test);
					});
			}).promise();
		}

		this.edit_stop = function(widget_name){
			return $.Deferred(function(df){
				df.resolveWith(self.test);
			}).promise();
		}

		this.set_n_columns = function(n){
			console.log('%cset_n_columns', 'color:#0a0', n);
			var self = this;
			if(n < 1 || n > 6) return test.fail('bad columns number:', n);

			var dashboard = $('.dashboard').dashboardSet('get_selected_dashboard2');
			var n_widgets = this.n_widgets();

			function check_n_columns(n){
				if(dashboard.options.dashboard.columns.length !== n) return false;
				if(dashboard.widgets.columns.length !== n) return false;
				return true;
			}

			function menu_item_is_selected(n){
				// n is zero indexed
				return $('.layout .menu-radio :checked').parent().index() === n;
			}

			return $.Deferred(function(df){
				test.set_timeout(10000);
				$('#layout-menu').click();
				test
					.wait_for('#menu-panes .layout .menu-list:visible', 'menu open')
					.pipe(function(){
						return $.Deferred(function(df2){
							var menu_item = '.layout li[title="' + n + ' Column' + (n === 1 ? '' : 's') + '"] input';
							test.wait_for(menu_item, 'menu item ' + n).then(function(){
								test.assert(!menu_item_is_selected(n - 1)); // click will not work if already set to n.
								$(menu_item).click();
								test.wait_for_remove('#menu-panes .layout .menu-list', 'menu close')
									.then(function(){
										test.assert(check_n_columns(n), 'incorrect number of columns');
										test.wait_for(function(){ return self.n_widgets() === n_widgets; }, 'incorrect number of widgets');
										test.wait_for(function(){ return menu_item_is_selected(n - 1); }, 'correct menu item not selected');
										df2.resolveWith(self.test);
									});
							})
						}).promise();
					})
					.then(function(){
						df.resolveWith(self.test);
					});
			}).promise();
		}
	}

	return DashboardTestable;
});
